#!/bin/sh

echo ""
echo "################################################################################"
echo "######################## bridge++ Makefile Setup script ########################"
echo "################################################################################"
#
echo ""

datestr=`date +%Y%m%d%H%M`
if [ -f Makefile ]; then
    echo "Current Makefile is saved to Makefile.$datestr."
    cp -p Makefile Makefile.$datestr
    echo ""
fi

#----------------------------------------------------------------
echo "### Select Platform ###"
echo "1 [PC WorkStation with GNU C++] (default)"
echo "2 [PC WorkStation with Intel C++]"
echo "3 [PC WorkStation with PGI compiler]"
echo "4 [SR16000@KEK]"
echo "5 [SR16000@YITP]"
echo "6 [BlueGene/Q@KEK]"
echo "7 [FX10@Tokyo]"
echo "8 [Others]"
echo ""

echo -n "Platform? > "
read VALUE

echo ""

case ${VALUE} in
    1)
	TARGET_FLAG="PC_GCC"
	echo "You have selected [PC Workstation with GNU C++]"
	;;
    2)
	TARGET_FLAG="PC_INTEL"
	echo "You have selected [PC Workstation with Intel C++]"
	;;
    3)
	TARGET_FLAG="PC_PGI"
	echo "You have selected [PC Workstation with PGI compiler]"
	;;
    4)
	TARGET_FLAG="KEKSR"
	echo "You have selected [SR16000@KEK]"
	;;
    5)
	TARGET_FLAG="YITPSR"
	echo "You have selected [SR16000@YITP]"
	;;
    6)
	TARGET_FLAG="KEKBGQ"
	echo "You have selected [BlueGene/Q@KEK]"
	;;
    7)
	TARGET_FLAG="FX10"
	echo "You have selected [FX10@Tokyo]"
	;;
    8)
	TARGET_FLAG="PC_GCC"
	echo "Edit Makefile for your own platform."
	;;
    *)
	TARGET_FLAG="PC_GCC"
	echo "You have selected default value [PC Workstation With GNU C++]"
	;;
esac

echo ""
#----------------------------------------------------------------
echo "### Select Communicator Type ###"
echo "## [BGNET] and [BGNET+MPI] are only for Blue Gene"
echo "1 [Single] (default)"
echo "2 [MPI]"
echo "3 [BGNET]"
echo "4 [BGNET+MPI]"
echo ""

echo -n "Communication Type? > "
read VALUE

echo ""

case ${VALUE} in
    1)
	MPI_FLAG="no"
	echo "You have selected [Single]"
	;;
    2)
	MPI_FLAG="yes"
	echo "You have selected [MPI]"
	;;
    3)
	MPI_FLAG="bgnet"
	echo "You have selected [BGNET]"
	;;
    4)
	MPI_FLAG="bgnet_mpi"
	echo "You have selected [BGNET+MPI]"
	;;
    *)
	MPI_FLAG="no"
	echo "You have selected default value [Single]"
	;;
esac

echo ""
#----------------------------------------------------------------
echo "### Select Multi-threading Option ###"
echo "1 [OpenMP]"
echo "2 [No] (default)"
echo ""

echo -n "Use Multi-threading? > "
read VALUE

echo ""

case ${VALUE} in
    1)
	MULTITHREAD_FLAG="omp"
	echo "You have selected [OpenMP]"
	;;
    2)
	MULTITHREAD_FLAG="no"
	echo "You have selected [No]"
	;;
    *)
	MULTITHREAD_FLAG="no"
	echo "You have selected default value [No]"
	;;
esac

echo ""
#----------------------------------------------------------------
echo "### Use Optimized Code ###"
echo "## NB. Don't set [No], if you use Multi-threading"
echo "1 [Yes] (default)"
echo "2 [No]"
echo "3 [enable BG/Q optimization]"
#echo "4 [enable BG/Q Wilson library]"
echo ""

echo -n "Use Optimized Code? > "
read VALUE

echo ""

case ${VALUE} in
    1)
	OPTIMIZE_FLAG="yes"
	echo "You have selected [Yes]"
	;;
    2)
	OPTIMIZE_FLAG="no"
	echo "You have selected [No]"
	;;
    3)
	OPTIMIZE_FLAG="imp_bgq"
	echo "You have selected [enable BG/Q optimization]"
	;;
#    4)
#	OPTIMIZE_FLAG="imp_bgwlib"
#	echo "You have selected [enable BG/Q Wilson library]"
#	;;
    *)
	OPTIMIZE_FLAG="yes"
	echo "You have selected default value [Yes]"
	;;
esac

echo ""
#----------------------------------------------------------------
echo "### Select Compiler Optimization Level ###"
echo "1 [Low]"
echo "2 [Standard] (default)"
echo "3 [High]"
echo ""

echo -n "Optimization Level? > "
read VALUE

echo ""

case ${VALUE} in
    1)
	OPT_LEVEL_FLAG="low"
	echo "You have selected [Low]"
	;;
    2)
	OPT_LEVEL_FLAG="std"
	echo "You have selected [Standard]"
	;;
    3)
	OPT_LEVEL_FLAG="high"
	echo "You have selected [High]"
	;;
    *)
	OPT_LEVEL_FLAG="std"
	echo "You have selected default value [Standard]"
	;;
esac

echo ""
#----------------------------------------------------------------
echo "### Use Static linkage Option ###"
echo "## NB. [Yes] may have a link error"
echo "1 [Yes]"
echo "2 [No] (default)"
echo ""

echo -n "Use Static linkage? > "
read VALUE

echo ""

case ${VALUE} in
    1)
	STATIC_FLAG="yes"
	echo "You have selected [Yes]"
	;;
    2)
	STATIC_FLAG="no"
	echo "You have selected [No]"
	;;
    *)
	STATIC_FLAG="no"
	echo "You have selected default value [No]"
	;;
esac

echo ""
#----------------------------------------------------------------
echo "### Use Debug Option ###"
echo "1 [Yes]"
echo "2 [No] (default)"
echo ""

echo -n "Use Debug Option? > "
read VALUE

echo ""

case ${VALUE} in
    1)
	DEBUG_FLAG="yes"
	echo "You have selected [Yes]"
	;;
    2)
	DEBUG_FLAG="no"
	echo "You have selected [No]"
	;;
    *)
	DEBUG_FLAG="no"
	echo "You have selected default value [No]"
	;;
esac

echo ""
#----------------------------------------------------------------
echo "### Select Gauge group ###"
echo "1 [SU(3)] (default)"
echo "2 [SU(2)]"
echo "3 [General SU(N)]"
echo ""

echo -n "Gauge group? > "
read VALUE

echo ""

case ${VALUE} in
    1)
	GAUGE_GROUP_FLAG="su3"
	echo "You have selected [SU(3)]"
	;;
    2)
	GAUGE_GROUP_FLAG="su2"
	echo "You have selected [SU(2)]"
	;;
    3)
	GAUGE_GROUP_FLAG="general"
	echo "You have selected [General SU(N)]"
	;;
    *)
	GAUGE_GROUP_FLAG="su3"
	echo "You have selected default value [SU(3)]"
	;;
esac

echo ""
#----------------------------------------------------------------
echo "### Select Complex Type ###"
echo "1 [STL complex] (default)"
echo "2 [C99 complex]"
echo ""

echo -n "Complex Type? > "
read VALUE

echo ""

case ${VALUE} in
    1)
	COMPLEX_FLAG="std"
	echo "You have selected [STL complex]"
	;;
    2)
	COMPLEX_FLAG="c99"
	echo "You have selected [C99 complex]"
	;;
    *)
	COMPLEX_FLAG="std"
	echo "You have selected default value [STL complex]"
	;;
esac

echo ""
#----------------------------------------------------------------
echo "### Use Test Manager ###"
echo "1 [Yes] (default)"
echo "2 [No]"
echo ""

echo -n "Use Test Manager? > "
read VALUE

echo ""

case ${VALUE} in
    1)
	TESTMANAGER_FLAG="yes"
	echo "You have selected [Yes]"
	;;
    2)
	TESTMANAGER_FLAG="no"
	echo "You have selected [No]"
	;;
    *)
	TESTMANAGER_FLAG="yes"
	echo "You have selected default value [Yes]"
	;;
esac

echo ""
#----------------------------------------------------------------
echo "### External Library Options ###"
echo ""

echo "### Use LIME library Option ###"
echo "1 [Yes]"
echo "2 [No] (default)"
echo ""

echo -n "Use LIME library? > "
read VALUE

echo ""

case ${VALUE} in
    1)
	USE_LIME_FLAG="yes"
	echo "You have selected [Yes]"
	;;
    2)
	USE_LIME_FLAG="no"
	echo "You have selected [No]"
	;;
    *)
	USE_LIME_FLAG="no"
	echo "You have selected default value [No]"
	;;
esac

echo ""
#----------------------------------------------------------------
echo "### Use SFMT(SIMD Fast Mersenne Twister) library Option ###"
echo "## NB. set [No] on Blue Gene, SR16000 and FX10"
echo "1 [Yes]"
echo "2 [No] (default)"
echo ""

echo -n "Use SFMT library? > "
read VALUE

echo ""

case ${VALUE} in
    1)
	USE_SFMT_FLAG="yes"
	echo "You have selected [Yes]"
	;;
    2)
	USE_SFMT_FLAG="no"
	echo "You have selected [No]"
	;;
    *)
	USE_SFMT_FLAG="no"
	echo "You have selected default value [No]"
	;;
esac

echo ""
#----------------------------------------------------------------
echo "### Use NTL library Option ###"
echo "## NB. set [No] on Blue Gene, SR16000 and FX10"
echo "1 [Yes]"
echo "2 [No] (default)"
echo ""

echo -n "Use NTL library? > "
read VALUE

echo ""

case ${VALUE} in
    1)
	USE_NTL_FLAG="yes"
	echo "You have selected [Yes]"
	;;
    2)
	USE_NTL_FLAG="no"
	echo "You have selected [No]"
	;;
    *)
	USE_NTL_FLAG="no"
	echo "You have selected default value [No]"
	;;
esac

echo ""
#----------------------------------------------------------------

# echo $TARGET_FLAG
# echo $MPI_FLAG
# echo $MULTITHREAD_FLAG
# echo $OPTIMIZE_FLAG
# echo $OPT_LEVEL_FLAG
# echo $STATIC_FLAG
# echo $DEBUG_FLAG
# echo $GAUGE_GROUP_FLAG
# echo $COMPLEX_FLAG
# echo $TESTMANAGER_FLAG
# echo $USE_LIME_FLAG
# echo $USE_SFMT_FLAG
# echo $USE_NTL_FLAG

sed \
    -e "s/@@TARGET@@/$TARGET_FLAG/" \
    -e "s/@@USE_MPI@@/$MPI_FLAG/" \
    -e "s/@@USE_THREAD@@/$MULTITHREAD_FLAG/" \
    -e "s/@@USE_OPTIMIZE@@/$OPTIMIZE_FLAG/" \
    -e "s/@@USE_OPT_LEVEL@@/$OPT_LEVEL_FLAG/" \
    -e "s/@@USE_STATIC@@/$STATIC_FLAG/" \
    -e "s/@@USE_DEBUG@@/$DEBUG_FLAG/" \
    -e "s/@@USE_GAUGE_GROUP@@/$GAUGE_GROUP_FLAG/" \
    -e "s/@@USE_COMPLEX@@/$COMPLEX_FLAG/" \
    -e "s/@@USE_TESTMANAGER@@/$TESTMANAGER_FLAG/" \
    -e "s/@@USE_LIME_LIBRARY@@/$USE_LIME_FLAG/" \
    -e "s/@@USE_SFMT_LIBRARY@@/$USE_SFMT_FLAG/" \
    -e "s/@@USE_NTL_LIBRARY@@/$USE_NTL_FLAG/" \
Makefile.in > Makefile

#----------------------------------------------------------------
echo "Makefile is generated."

exit 0

