/*!
        @file    $Id:: field_G_imp.cpp #$

        @brief

        @author  <Hideo Matsufuru> hideo.matsufuru@kek.jp(matsufuru)
                 $LastChangedBy: aoym $

        @date    $LastChangedDate:: 2014-09-16 12:23:39 #$

        @version $LastChangedRevision: 1144 $
*/

#include "field_G.h"

#include "bridgeIO.h"
using Bridge::vout;

//#include "threadManager_OpenMP.h"

#if defined USE_GROUP_SU3
#include "field_G_imp_SU3.inc"
#elif defined USE_GROUP_SU2
#include "field_G_imp_SU2.inc"
#elif defined USE_GROUP_SU_N
#include "field_G_imp_SU_N.inc"
#endif

//====================================================================
void Field_G::check()
{
  assert(NC == CommonParameters::Nc());

  check_Nc();
}


//====================================================================
void Field_G::set_unit()
{
  Mat_SU_N ut(nc());

  ut.unit();

  //  int nth = ThreadManager_OpenMP::get_num_threads();
  //  int ith = ThreadManager_OpenMP::get_thread_id();
  int nth = 1;
  int ith = 0;

  int is = nvol() * ith / nth;
  int ns = nvol() * (ith + 1) / nth - is;

  for (int mu = 0; mu < nex(); ++mu) {
    for (int site = is; site < is + ns; ++site) {
      set_mat(site, mu, ut);
    }
  }
}


//====================================================================
void Field_G::set_random(RandomNumbers *rand)
{
  rand->gauss_lex_global(*this);

  Mat_SU_N ut(m_Nc);

  for (int mu = 0; mu < m_Nex; ++mu) {
    for (int site = 0; site < m_Nvol; ++site) {
      this->mat(ut, site, mu);
      ut.reunit();
      this->set_mat(site, mu, ut);
    }
  }
}


//====================================================================
void Field_G::reunit()
{
  Mat_SU_N ut(nc());

  //  int nth = ThreadManager_OpenMP::get_num_threads();
  //  int ith = ThreadManager_OpenMP::get_thread_id();
  int nth = 1;
  int ith = 0;

  int is = nvol() * ith / nth;
  int ns = nvol() * (ith + 1) / nth - is;

  for (int mu = 0; mu < nex(); ++mu) {
    for (int site = is; site < is + ns; ++site) {
      mat(ut, site, mu);
      ut.reunit();
      set_mat(site, mu, ut);
    }
  }
}


//====================================================================
void mult_Field_Gnn(Field_G& w, const int ex,
                    const Field_G& u1, int ex1,
                    const Field_G& u2, int ex2)
{
  assert(ex < w.nex());
  assert(ex1 < u1.nex());
  assert(ex2 < u2.nex());
  assert(u1.nvol() == w.nvol());
  assert(u2.nvol() == w.nvol());

  //  int nth = ThreadManager_OpenMP::get_num_threads();
  //  int ith = ThreadManager_OpenMP::get_thread_id();
  int nth = 1;
  int ith = 0;

  int is = w.nvol() * ith / nth;
  int ns = w.nvol() * (ith + 1) / nth - is;

  double *g  = w.ptr(0, is, ex);
  double *g1 = const_cast<Field_G *>(&u1)->ptr(0, is, ex1);
  double *g2 = const_cast<Field_G *>(&u2)->ptr(0, is, ex2);

  const int Nc  = w.nc();
  const int Nc2 = 2 * Nc;
  const int Ndf = Nc2 * Nc;

  for (int site = 0; site < ns; ++site) {
    int ig = Ndf * site;

    for (int ic2 = 0; ic2 < Nc; ++ic2) {
      for (int ic1 = 0; ic1 < Nc; ++ic1) {
        int jg2 = ic2 * 2 + ig;
        int jg1 = ic1 * Nc2 + ig;
        g[2 * ic2 + Nc2 * ic1 + ig]     = mult_Gnn_r(&g1[jg1], &g2[jg2], Nc);
        g[2 * ic2 + 1 + Nc2 * ic1 + ig] = mult_Gnn_i(&g1[jg1], &g2[jg2], Nc);
      }
    }
  }
}


//====================================================================
void mult_Field_Gdn(Field_G& w, const int ex,
                    const Field_G& u1, int ex1,
                    const Field_G& u2, int ex2)
{
  assert(ex < w.nex());
  assert(ex1 < u1.nex());
  assert(ex2 < u2.nex());
  assert(u1.nvol() == w.nvol());
  assert(u2.nvol() == w.nvol());

  //  int nth = ThreadManager_OpenMP::get_num_threads();
  //  int ith = ThreadManager_OpenMP::get_thread_id();
  int nth = 1;
  int ith = 0;

  int is = w.nvol() * ith / nth;
  int ns = w.nvol() * (ith + 1) / nth - is;

  double *g  = w.ptr(0, is, ex);
  double *g1 = const_cast<Field_G *>(&u1)->ptr(0, is, ex1);
  double *g2 = const_cast<Field_G *>(&u2)->ptr(0, is, ex2);

  const int Nc  = w.nc();
  const int Nc2 = 2 * Nc;
  const int Ndf = Nc2 * Nc;

  for (int site = 0; site < ns; ++site) {
    int ig = Ndf * site;

    for (int ic2 = 0; ic2 < Nc; ++ic2) {
      for (int ic1 = 0; ic1 < Nc; ++ic1) {
        int jg2 = ic2 * 2 + ig;
        int jg1 = ic1 * 2 + ig;
        g[2 * ic2 + Nc2 * ic1 + ig]     = mult_Gdn_r(&g1[jg1], &g2[jg2], Nc);
        g[2 * ic2 + 1 + Nc2 * ic1 + ig] = mult_Gdn_i(&g1[jg1], &g2[jg2], Nc);
      }
    }
  }
}


//====================================================================
void mult_Field_Gnd(Field_G& w, const int ex,
                    const Field_G& u1, int ex1,
                    const Field_G& u2, int ex2)
{
  assert(ex < w.nex());
  assert(ex1 < u1.nex());
  assert(ex2 < u2.nex());
  assert(u1.nvol() == w.nvol());
  assert(u2.nvol() == w.nvol());

  //  int nth = ThreadManager_OpenMP::get_num_threads();
  //  int ith = ThreadManager_OpenMP::get_thread_id();
  int nth = 1;
  int ith = 0;

  int is = w.nvol() * ith / nth;
  int ns = w.nvol() * (ith + 1) / nth - is;

  double *g  = w.ptr(0, is, ex);
  double *g1 = const_cast<Field_G *>(&u1)->ptr(0, is, ex1);
  double *g2 = const_cast<Field_G *>(&u2)->ptr(0, is, ex2);

  const int Nc  = w.nc();
  const int Nc2 = 2 * Nc;
  const int Ndf = Nc2 * Nc;

  for (int site = 0; site < ns; ++site) {
    int ig = Ndf * site;

    for (int ic2 = 0; ic2 < Nc; ++ic2) {
      for (int ic1 = 0; ic1 < Nc; ++ic1) {
        int jg2 = ic2 * Nc2 + ig;
        int jg1 = ic1 * Nc2 + ig;
        g[2 * ic2 + Nc2 * ic1 + ig]     = mult_Gnd_r(&g1[jg1], &g2[jg2], Nc);
        g[2 * ic2 + 1 + Nc2 * ic1 + ig] = mult_Gnd_i(&g1[jg1], &g2[jg2], Nc);
      }
    }
  }
}


//====================================================================
void mult_Field_Gdd(Field_G& w, const int ex,
                    const Field_G& u1, int ex1,
                    const Field_G& u2, int ex2)
{
  assert(ex < w.nex());
  assert(ex1 < u1.nex());
  assert(ex2 < u2.nex());
  assert(u1.nvol() == w.nvol());
  assert(u2.nvol() == w.nvol());

  //  int nth = ThreadManager_OpenMP::get_num_threads();
  //  int ith = ThreadManager_OpenMP::get_thread_id();
  int nth = 1;
  int ith = 0;

  int is = w.nvol() * ith / nth;
  int ns = w.nvol() * (ith + 1) / nth - is;

  double *g  = w.ptr(0, is, ex);
  double *g1 = const_cast<Field_G *>(&u1)->ptr(0, is, ex1);
  double *g2 = const_cast<Field_G *>(&u2)->ptr(0, is, ex2);

  const int Nc  = w.nc();
  const int Nc2 = 2 * Nc;
  const int Ndf = Nc2 * Nc;

  for (int site = 0; site < ns; ++site) {
    int ig = Ndf * site;

    for (int ic2 = 0; ic2 < Nc; ++ic2) {
      for (int ic1 = 0; ic1 < Nc; ++ic1) {
        int jg2 = ic2 * Nc2 + ig;
        int jg1 = ic1 * 2 + ig;
        g[2 * ic2 + Nc2 * ic1 + ig]     = mult_Gdd_r(&g1[jg1], &g2[jg2], Nc);
        g[2 * ic2 + 1 + Nc2 * ic1 + ig] = mult_Gdd_i(&g1[jg1], &g2[jg2], Nc);
      }
    }
  }
}


//====================================================================
void multadd_Field_Gnn(Field_G& w, const int ex,
                       const Field_G& u1, int ex1,
                       const Field_G& u2, int ex2,
                       const double ff)
{
  assert(ex < w.nex());
  assert(ex1 < u1.nex());
  assert(ex2 < u2.nex());
  assert(u1.nvol() == w.nvol());
  assert(u2.nvol() == w.nvol());

  //  int nth = ThreadManager_OpenMP::get_num_threads();
  //  int ith = ThreadManager_OpenMP::get_thread_id();
  int nth = 1;
  int ith = 0;

  int is = w.nvol() * ith / nth;
  int ns = w.nvol() * (ith + 1) / nth - is;

  double *g  = w.ptr(0, is, ex);
  double *g1 = const_cast<Field_G *>(&u1)->ptr(0, is, ex1);
  double *g2 = const_cast<Field_G *>(&u2)->ptr(0, is, ex2);

  const int Nc  = w.nc();
  const int Nc2 = 2 * Nc;
  const int Ndf = Nc2 * Nc;

  for (int site = 0; site < ns; ++site) {
    int ig = Ndf * site;

    for (int ic2 = 0; ic2 < Nc; ++ic2) {
      for (int ic1 = 0; ic1 < Nc; ++ic1) {
        int jg2 = ic2 * 2 + ig;
        int jg1 = ic1 * Nc2 + ig;
        g[2 * ic2 + Nc2 * ic1 + ig] +=
          ff * mult_Gnn_r(&g1[jg1], &g2[jg2], Nc);
        g[2 * ic2 + 1 + Nc2 * ic1 + ig] +=
          ff * mult_Gnn_i(&g1[jg1], &g2[jg2], Nc);
      }
    }
  }
}


//====================================================================
void multadd_Field_Gdn(Field_G& w, const int ex,
                       const Field_G& u1, int ex1,
                       const Field_G& u2, int ex2,
                       const double ff)
{
  assert(ex < w.nex());
  assert(ex1 < u1.nex());
  assert(ex2 < u2.nex());
  assert(u1.nvol() == w.nvol());
  assert(u2.nvol() == w.nvol());

  //  int nth = ThreadManager_OpenMP::get_num_threads();
  //  int ith = ThreadManager_OpenMP::get_thread_id();
  int nth = 1;
  int ith = 0;

  int is = w.nvol() * ith / nth;
  int ns = w.nvol() * (ith + 1) / nth - is;

  double *g  = w.ptr(0, is, ex);
  double *g1 = const_cast<Field_G *>(&u1)->ptr(0, is, ex1);
  double *g2 = const_cast<Field_G *>(&u2)->ptr(0, is, ex2);

  const int Nc  = w.nc();
  const int Nc2 = 2 * Nc;
  const int Ndf = Nc2 * Nc;

  for (int site = 0; site < ns; ++site) {
    int ig = Ndf * site;

    for (int ic2 = 0; ic2 < Nc; ++ic2) {
      for (int ic1 = 0; ic1 < Nc; ++ic1) {
        int jg2 = ic2 * 2 + ig;
        int jg1 = ic1 * 2 + ig;
        g[2 * ic2 + Nc2 * ic1 + ig] +=
          ff * mult_Gdn_r(&g1[jg1], &g2[jg2], Nc);
        g[2 * ic2 + 1 + Nc2 * ic1 + ig] +=
          ff * mult_Gdn_i(&g1[jg1], &g2[jg2], Nc);
      }
    }
  }
}


//====================================================================
void multadd_Field_Gnd(Field_G& w, const int ex,
                       const Field_G& u1, int ex1,
                       const Field_G& u2, int ex2,
                       const double ff)
{
  assert(ex < w.nex());
  assert(ex1 < u1.nex());
  assert(ex2 < u2.nex());
  assert(u1.nvol() == w.nvol());
  assert(u2.nvol() == w.nvol());

  //  int nth = ThreadManager_OpenMP::get_num_threads();
  //  int ith = ThreadManager_OpenMP::get_thread_id();
  int nth = 1;
  int ith = 0;

  int is = w.nvol() * ith / nth;
  int ns = w.nvol() * (ith + 1) / nth - is;

  double *g  = w.ptr(0, is, ex);
  double *g1 = const_cast<Field_G *>(&u1)->ptr(0, is, ex1);
  double *g2 = const_cast<Field_G *>(&u2)->ptr(0, is, ex2);

  const int Nc  = w.nc();
  const int Nc2 = 2 * Nc;
  const int Ndf = Nc2 * Nc;

  for (int site = 0; site < ns; ++site) {
    int ig = Ndf * site;

    for (int ic2 = 0; ic2 < Nc; ++ic2) {
      for (int ic1 = 0; ic1 < Nc; ++ic1) {
        int jg2 = ic2 * Nc2 + ig;
        int jg1 = ic1 * Nc2 + ig;
        g[2 * ic2 + Nc2 * ic1 + ig] +=
          ff * mult_Gnd_r(&g1[jg1], &g2[jg2], Nc);
        g[2 * ic2 + 1 + Nc2 * ic1 + ig]
          += ff * mult_Gnd_i(&g1[jg1], &g2[jg2], Nc);
      }
    }
  }
}


//====================================================================
void multadd_Field_Gdd(Field_G& w, const int ex,
                       const Field_G& u1, int ex1,
                       const Field_G& u2, int ex2,
                       const double ff)
{
  assert(ex < w.nex());
  assert(ex1 < u1.nex());
  assert(ex2 < u2.nex());
  assert(u1.nvol() == w.nvol());
  assert(u2.nvol() == w.nvol());

  //  int nth = ThreadManager_OpenMP::get_num_threads();
  //  int ith = ThreadManager_OpenMP::get_thread_id();
  int nth = 1;
  int ith = 0;

  int is = w.nvol() * ith / nth;
  int ns = w.nvol() * (ith + 1) / nth - is;

  double *g  = w.ptr(0, is, ex);
  double *g1 = const_cast<Field_G *>(&u1)->ptr(0, is, ex1);
  double *g2 = const_cast<Field_G *>(&u2)->ptr(0, is, ex2);

  const int Nc  = w.nc();
  const int Nc2 = 2 * Nc;
  const int Ndf = Nc2 * Nc;

  for (int site = 0; site < ns; ++site) {
    int ig = Ndf * site;

    for (int ic2 = 0; ic2 < Nc; ++ic2) {
      for (int ic1 = 0; ic1 < Nc; ++ic1) {
        int jg2 = ic2 * Nc2 + ig;
        int jg1 = ic1 * 2 + ig;
        g[2 * ic2 + Nc2 * ic1 + ig] +=
          ff * mult_Gdd_r(&g1[jg1], &g2[jg2], Nc);
        g[2 * ic2 + 1 + Nc2 * ic1 + ig] +=
          ff * mult_Gdd_i(&g1[jg1], &g2[jg2], Nc);
      }
    }
  }
}


//====================================================================
void at_Field_G(Field_G& w, const int ex)
{
  assert(ex < w.nex());

  //  int nth = ThreadManager_OpenMP::get_num_threads();
  //  int ith = ThreadManager_OpenMP::get_thread_id();
  int nth = 1;
  int ith = 0;

  int is = w.nvol() * ith / nth;
  int ns = w.nvol() * (ith + 1) / nth - is;

  double *g = w.ptr(0, is, ex);

  const int Nc  = w.nc();
  const int Ndf = 2 * Nc * Nc;

  for (int site = 0; site < ns; ++site) {
    int ig = Ndf * site;

    for (int a = 0; a < Nc; ++a) {
      for (int b = a + 1; b < Nc; ++b) {
        double re = g[2 * (Nc * a + b) + ig] - g[2 * (Nc * b + a) + ig];
        double im = g[2 * (Nc * a + b) + 1 + ig] + g[2 * (Nc * b + a) + 1 + ig];

        g[2 * (Nc * a + b) + ig]     = 0.5 * re;
        g[2 * (Nc * a + b) + 1 + ig] = 0.5 * im;

        g[2 * (Nc * b + a) + ig]     = -0.5 * re;
        g[2 * (Nc * b + a) + 1 + ig] = 0.5 * im;
      }
    }
    double tr = 0.0;
    for (int cc = 0; cc < Nc; ++cc) {
      tr += g[2 * (Nc * cc + cc) + 1 + ig];
    }
    tr = tr / Nc;
    for (int cc = 0; cc < Nc; ++cc) {
      g[2 * (Nc * cc + cc) + ig]      = 0.0;
      g[2 * (Nc * cc + cc) + 1 + ig] -= tr;
    }
  }
}


//====================================================================
void ah_Field_G(Field_G& w, const int ex)
{
  assert(ex < w.nex());

  //  int nth = ThreadManager_OpenMP::get_num_threads();
  //  int ith = ThreadManager_OpenMP::get_thread_id();
  int nth = 1;
  int ith = 0;

  int is = w.nvol() * ith / nth;
  int ns = w.nvol() * (ith + 1) / nth - is;

  double *g = w.ptr(0, is, ex);

  const int Nc  = w.nc();
  const int Ndf = 2 * Nc * Nc;

  for (int site = 0; site < ns; ++site) {
    int ig = Ndf * site;

    for (int a = 0; a < Nc; ++a) {
      for (int b = a; b < Nc; ++b) {
        double re = g[2 * (Nc * a + b) + ig] - g[2 * (Nc * b + a) + ig];
        double im = g[2 * (Nc * a + b) + 1 + ig] + g[2 * (Nc * b + a) + 1 + ig];

        g[2 * (Nc * a + b) + ig]     = 0.5 * re;
        g[2 * (Nc * a + b) + 1 + ig] = 0.5 * im;

        g[2 * (Nc * b + a) + ig]     = -0.5 * re;
        g[2 * (Nc * b + a) + 1 + ig] = 0.5 * im;
      }
    }
  }
}


//====================================================================
//============================================================END=====
