/*!
        @file    $Id:: fopr_Wilson_eo_impl.h #$

        @brief

        @author  UEDA, Satoru <sueda@post.kek.jp>
                 $LastChangedBy: namekawa $

        @date    $LastChangedDate:: 2014-09-16 18:27:22 #$

        @version $LastChangedRevision: 1145 $
*/


#ifndef FOPR_WILSON_EO_IMPL_INCLUDED
#define FOPR_WILSON_EO_IMPL_INCLUDED

#include "fopr_Wilson_eo.h"

//#include "fopr_eo.h"
#include "shiftField_eo.h"

#include "gammaMatrixSet.h"

#include "bridgeIO.h"
using Bridge::vout;

//! Implementation of even-odd Wilson fermion operator.

/*!
    This class is a subclass of Fopr_Wilson_eo and implements
    an even-odd version of the Wilson fermion operator.
    This is rather straightforward and readable while slower
    version that was coded by S.Ueda [20 Jun 2012 S.UEDA].
    The implementation class was separated.
                                     [07 Jul 2014 H.Matsufuru]
 */

class Fopr_Wilson_eo::Fopr_Wilson_eo_impl
{
 public:
  static const std::string class_name;

 private:
  int                m_Nvol, m_Nvol2, m_Ndim;
  int                m_Nc, m_Nd;
  double             m_kappa;
  std::valarray<int> m_boundary;
  Index_eo           m_index;
  Field_G            *m_Ueo;

  ShiftField_eo shift;
  Field_F       trf, trf2;
  Vec_SU_N      v1, v2;

  std::valarray<GammaMatrix> m_GM;

  Bridge::VerboseLevel m_vl;

  std::string m_mode;
  std::string m_repr;

 public:

  Fopr_Wilson_eo_impl(std::string repr)
    : m_Nvol(CommonParameters::Nvol()),
      m_Nvol2(CommonParameters::Nvol() / 2),
      m_Ndim(CommonParameters::Ndim()),
      m_Nc(CommonParameters::Nc()),
      m_Nd(CommonParameters::Nd()),
      trf(m_Nvol2, 1), trf2(m_Nvol2, 1)
  {
    init(repr);
  }

  ~Fopr_Wilson_eo_impl()
  {
    delete m_Ueo;
  }

  void set_parameters(const double kappa, const std::valarray<int> bc);

  void set_config(Field *U);

  void prePropD(Field&, Field&, const Field&);
  void postPropD(Field&, const Field&, const Field&);
  void prePropDag(Field&, Field&, const Field&);
  void postPropDag(Field&, const Field&, const Field&);

  void D(Field& v, const Field& f);
  void Ddag(Field& v, const Field& f);
  void DdagD(Field& v, const Field& f);
  void DDdag(Field& v, const Field& f);
  void H(Field& v, const Field& f);

  // ieo=0: even <-- odd
  // ieo=1: odd  <-- even

  void Meo(Field&, const Field&, const int ieo);
  void Mdageo(Field&, const Field&, const int ieo);
  void MeoMoe(Field& v, const Field& f);
  void Meo_gm5(Field&, const Field&, const int ieo);

  void mult_gm5(Field&, const Field&);

  //! gamma_5 (1 - gamma_mu) v(x + mu)
  void gm5p(const int mu, Field&, const Field& v);

  int field_nvol() { return CommonParameters::Nvol() / 2; }
  int field_nin() { return 2 * CommonParameters::Nc() * CommonParameters::Nd(); }
  int field_nex() { return 1; }

  //! this returns the number of floating point operations of Meo.
  double flop_count();

  void mult_p(int mu, Field_F&, const Field_F&, const int ieo);
  void mult_m(int mu, Field_F&, const Field_F&, const int ieo);

 private:
  void init(const std::string);

  /*
  void mult_xp(Field_F&, const Field_F&, const int ieo);
  void mult_xm(Field_F&, const Field_F&, const int ieo);
  void mult_yp(Field_F&, const Field_F&, const int ieo);
  void mult_ym(Field_F&, const Field_F&, const int ieo);
  void mult_zp(Field_F&, const Field_F&, const int ieo);
  void mult_zm(Field_F&, const Field_F&, const int ieo);
  void mult_tp(Field_F&, const Field_F&, const int ieo);
  void mult_tm(Field_F&, const Field_F&, const int ieo);
  */
};
#endif
