/*!
        @file    $Id:: force_F_Wilson_Nf2_Isochemical.h #$

        @brief

        @author  <Hideo Matsufuru> hideo.matsufuru@kek.jp(matsufuru)
                 $LastChangedBy: aoym $

        @date    $LastChangedDate:: 2014-06-06 17:58:37 #$

        @version $LastChangedRevision: 1063 $
*/

#ifndef FORCE_F_WILSON_NF2_ISOCHEMICAL_INCLUDED
#define FORCE_F_WILSON_NF2_ISOCHEMICAL_INCLUDED

#include "force.h"
#include "fopr_Wilson_Isochemical.h"

#include "tensorProd.h"

#include "bridgeIO.h"
using Bridge::vout;

//! Force for the Wilson fermion operator with isospin chemical potential.

/*!
    This class calculate the force of the standard Wilson fermion with
    isospin chemical potential with two flavors.
                                     [24 Aug 2011 H.Matusfuru]
    (Coding history will be recovered from trac.)
    YAML is implemented.             [14 Nov 2012 Y.Namekawa]
 */

//- parameters class
class Parameters_Force_F_Wilson_Nf2_Isochemical : virtual public Parameters
{
 public:
  Parameters_Force_F_Wilson_Nf2_Isochemical();
};
//- end

class Force_F_Wilson_Nf2_Isochemical : public Force
{
 public:
  static const std::string class_name;

 private:
//  Field_G *m_U;

  double                  m_kappa;     //!< hopping parameter
  double                  m_mu;        //!< isospin chemical potential
  double                  m_exp_mu;    //!< exp(mu)
  std::valarray<int>      m_boundary;
  Fopr_Wilson_Isochemical *m_fopr_w;
  Field_F                 m_psf;

  std::string m_repr;
  std::string m_mode;

 public:

  Force_F_Wilson_Nf2_Isochemical()
    : Force()
  {
    m_repr   = "Dirac";
    m_fopr_w = new Fopr_Wilson_Isochemical(m_repr);
    m_boundary.resize(CommonParameters::Ndim());
  }

  Force_F_Wilson_Nf2_Isochemical(std::string repr)
    : Force()
  {
    m_repr   = repr;
    m_fopr_w = new Fopr_Wilson_Isochemical(m_repr);
    m_boundary.resize(CommonParameters::Ndim());
  }

  ~Force_F_Wilson_Nf2_Isochemical()
  {
    delete m_fopr_w;
  }

  void set_parameters(const Parameters& params);

  // void set_parameters (const Parameters_Fopr_Wilson_Isochemical& params);
  void set_parameters(const double kappa, const double mu, const std::valarray<int> bc);

  void set_config(Field *U)
  {
    m_U = (Field_G *)U;
    m_fopr_w->set_config(U);
  }

  void set_mode(const std::string& mode)
  {
    m_mode = mode;
  }

//  void force_core(Field& force, const Field& eta);
//  void force_core1(Field& force, const Field& zeta, const Field& eta);

  void force_udiv(Field& force, const Field& eta);
  void force_udiv1(Field& force, const Field& zeta, const Field& eta);

 private:
  void force_udiv1_impl(Field_G& force, const Field_F& zeta, const Field_F& eta);
};
#endif
