/*!
        @file    $Id:: source_4spinor_Local.h #$

        @brief

        @author  <Hideo Matsufuru> hideo.matsufuru@kek.jp(matsufuru)
                 $LastChangedBy: aoym $

        @date    $LastChangedDate:: 2014-04-12 16:31:14 #$

        @version $LastChangedRevision: 1031 $
*/

#ifndef SOURCE_4SPINOR_LOCAL_INCLUDED
#define SOURCE_4SPINOR_LOCAL_INCLUDED

#include "field_F.h"
#include "index_lex.h"
#include "parameters.h"

#include "bridgeIO.h"
using Bridge::vout;

//! Local source for 4-spinor fermion.

/*!
    This class sets a local source vector for the 4-spinor
    (Wilson-type) fermion.
                                 [19 Feb 2012 H.Matsufuru]
    (Coding history will be recovered from trac.)
    YAML is implemented.         [14 Nov 2012 Y.Namekawa]
 */

//- parameters class
class Parameters_Source_4spinor_Local : virtual public Parameters
{
 public:
  Parameters_Source_4spinor_Local();
};
//- end

class Source_4spinor_Local
{
 public:
  static const std::string class_name;

 protected:
  Bridge::VerboseLevel m_vl;

 private:
  Index_lex          m_index;
  std::valarray<int> m_source_position;

 public:

  Source_4spinor_Local()
    : m_vl(CommonParameters::Vlevel()) {}

  void set_parameters(const Parameters& params);
  void set_parameters(std::valarray<int>& source_position);

  void set_parameter_verboselevel(const Bridge::VerboseLevel vl) { m_vl = vl; }

  void set(Field_F& src, int ic, int id);
};
#endif
