/*!
        @file    $Id:: topologicalCharge.h #$

        @brief

        @author  Yusuke Namekawa <namekawa@ccs.tsukuba.ac.jp> (namekawa)
                 $LastChangedBy: aoym $

        @date    $LastChangedDate:: 2014-09-16 12:23:39 #$

        @version $LastChangedRevision: 1144 $
*/

#ifndef TOPOLOGICALCHARGE_INCLUDED
#define TOPOLOGICALCHARGE_INCLUDED

#include "staples.h"
#include "shiftField_lex.h"

#include "bridgeIO.h"
using Bridge::vout;

//! Topological Charge measurement.

/*!
    This class measures a topological charge
    defined by a clover leaf on the lattice.
                     [01 Jan 2014 Y.Namekawa]
 */

//- parameters class
class Parameters_TopologicalCharge : virtual public Parameters
{
 public:
  Parameters_TopologicalCharge();
};
//- end

class TopologicalCharge
{
 public:
  static const std::string class_name;

 protected:
  Bridge::VerboseLevel m_vl;


 private:
  double m_c_plaq;
  double m_c_rect;

  ShiftField_lex m_shift;


 public:
  TopologicalCharge()
    : m_vl(CommonParameters::Vlevel()) {}

  virtual ~TopologicalCharge() {}

  //! setting parameters.
  virtual void set_parameters(const Parameters& params);

  void set_parameters(double c_plaq, double c_rect);

  void set_parameter_verboselevel(const Bridge::VerboseLevel vl) { m_vl = vl; }

  //! main function to measure Topological Charge.
  double measure(Field_G& U);


 private:
  void construct_Fmunu_1x1(Field_G& Fmunu, int mu, int nu, Field_G& U);
  void construct_Fmunu_1x2(Field_G& Fmunu, int mu, int nu, Field_G& U);

  double contract_epsilon_tensor(Field_G& Fmunu_1, Field_G& Fmunu_2);
};
#endif
