/*!
        @file    $Id:: solver.h #$

        @brief

        @author  <Hideo Matsufuru> hideo.matsufuru@kek.jp(matsufuru)
                 $LastChangedBy: aoym $

        @date    $LastChangedDate:: 2014-09-16 12:23:39 #$

        @version $LastChangedRevision: 1144 $
*/

#ifndef SOLVER_INCLUDED
#define SOLVER_INCLUDED

#include "defs.h"
#include "fopr.h"

#include "threadManager_OpenMP.h"

#ifdef USE_FACTORY
#include "factory.h"
#endif

#ifdef USE_PARAMETERS_FACTORY
#include "parameters_factory.h"
#endif


//! Base class for linear solver class family.

/*!
                               [28 Dec 2011 H.Matsufuru]
 */

class Solver
{
 public:
  Solver()
    : m_vl(CommonParameters::Vlevel()) {}

  virtual ~Solver() {}

  virtual void set_parameters(const Parameters& params) = 0;
  virtual void set_parameters(const int Niter, const double Stop_cond) = 0;

  void set_parameter_verboselevel(const Bridge::VerboseLevel vl) { m_vl = vl; }

  virtual void solve(Field& solution, const Field& source,
                     int& Nconv, double& diff) = 0;

  virtual Fopr *get_fopr() = 0;

 protected:
  Bridge::VerboseLevel m_vl;

#ifdef USE_FACTORY
 public:
  typedef Solver *(*ProductCreator)(Fopr *);
  typedef FactoryTemplate<Solver, ProductCreator>   Factory;

  static Solver *New(const IdentifierType& subtype, Fopr *fopr)
  {
    ProductCreator p = Factory::Find(subtype);

    return p ? (*p)(fopr) : 0;
  }

#endif
};
#endif
