/*!
        @file    $Id:: solver_BiCGStab_Cmplx.h #$

        @brief

        @author  <Yusuke Namekawa> namekawa@ccs.tsukuba.ac.jp(namekawa)
                 $LastChangedBy: aoym $

        @date    $LastChangedDate:: 2014-09-16 12:23:39 #$

        @version $LastChangedRevision: 1144 $
*/

#ifndef SOLVER_BICGSTAB_CMPLX_INCLUDED
#define SOLVER_BICGSTAB_CMPLX_INCLUDED

#include "solver.h"

#include "bridgeIO.h"
using Bridge::vout;


//- parameters class
class Parameters_Solver_BiCGStab_Cmplx : virtual public Parameters
{
 public:
  Parameters_Solver_BiCGStab_Cmplx();
};
//- end

//! BiCGStab algorithm with complex variables.

/*!
   This class implements BiCGStab algorithm for nonhermitian matrix.
   The product of vectors is treated in complex.
                                   [12 Feb 2012 Y.Namekawa]
    (Coding history will be recovered from trac.)
    YAML is implemented.           [14 Nov 2012 Y.Namekawa]
    Multi-threaded.                [10 Jul 2014 H.Matsufuru]
 */

class Solver_BiCGStab_Cmplx : public Solver
{
 public:
  static const std::string class_name;

 private:
  Fopr   *m_fopr;
  int    m_Niter;
  double m_Stop_cond;

  Field    s, r, x, rh, p, v, t;
  dcomplex rho_prev, alpha_prev, omega_prev;

 public:
  Solver_BiCGStab_Cmplx(Fopr *fopr)
    : Solver(), m_fopr(fopr) {}

  ~Solver_BiCGStab_Cmplx() {}

  void set_parameters(const Parameters& params);
  void set_parameters(const int Niter, const double Stop_cond);

  void solve(Field& solution, const Field& source,
             int& Nconv, double& diff);

  Fopr *get_fopr() { return m_fopr; }

 private:
  void reset_field(const Field&);

  void solve_init(const Field&, double&);
  void solve_step(double&);
};
#endif
