/*!
        @file    $Id:: solver_BiCGStab_IDS_L_Cmplx.h #$

        @brief

        @author  <Yusuke Namekawa> namekawa@ccs.tsukuba.ac.jp(namekawa)
                 $LastChangedBy: aoym $

        @date    $LastChangedDate:: 2014-09-16 12:23:39 #$

        @version $LastChangedRevision: 1144 $
*/

#ifndef SOLVER_BICGSTAB_IDS_L_CMPLX_INCLUDED
#define SOLVER_BICGSTAB_IDS_L_CMPLX_INCLUDED

#include "solver.h"

#include "bridgeIO.h"
using Bridge::vout;

//- parameters class
class Parameters_Solver_BiCGStab_IDS_L_Cmplx : virtual public Parameters
{
 public:
  Parameters_Solver_BiCGStab_IDS_L_Cmplx();
};
//- end

//! BiCGStab(IDS_L) algorithm.

/*!
    This class implements BiCGStab(IDS_L) algorithm for nonhermitian matrix.
    The product of vectors is treated in complex.
    See S.Itoh and Y.Namekawa, J.Comp.Appl.Math. 159 (2003) 65.
                                   [22 Jan 2012 Y.Namekawa]
    (Coding history will be recovered from trac.)
    YAML is implemented.           [14 Nov 2012 Y.Namekawa]
    Multi-threaded.                [17 Jul 2014 Y.Namekawa]
 */

class Solver_BiCGStab_IDS_L_Cmplx : public Solver
{
 public:
  static const std::string class_name;

 private:
  Fopr *m_fopr;

  int    m_N_L;
  double m_Tol_L;

  int    m_Niter;
  double m_Stop_cond;

  std::valarray<Field> u, r;

  Field s, x, r_init, v_tmp;

  dcomplex rho_prev, alpha_prev;
  double   rr_prev;
  int      N_L_prev;

 public:
  Solver_BiCGStab_IDS_L_Cmplx(Fopr *fopr)
    : Solver(), m_fopr(fopr) {}

  ~Solver_BiCGStab_IDS_L_Cmplx() {}

  void set_parameters(const Parameters& params);
  void set_parameters(const int Niter, const double Stop_cond);
  void set_parameters_DS_L(const int N_L, const double Tol_L);

  void solve(Field& solution, const Field& source,
             int& Nconv, double& diff);

  Fopr *get_fopr() { return m_fopr; }

 private:
  void reset_field(const Field&);

  void solve_init(const Field&, double&);
  void solve_step(double&);

  int index_ij(int i, int j)
  {
    return i + m_N_L * j;
  }
};
#endif
