/*!
        @file    $Id: solver_CGNR.cpp #$

        @brief

        @author  Tatsumi Aoyama <aoym@post.kek.jp> (aoym)
                 $LastChangedBy: aoym $

        @date    $LastChangedDate: 2013-04-27 12:28:50 #$

        @version $LastChangedRevision: 1144 $
*/

#include "solver_CGNR.h"

using Bridge::vout;


#ifdef USE_FACTORY
namespace {
  Solver *create_object(Fopr *fopr)
  {
    return new Solver_CGNR(fopr);
  }


  bool init = Solver::Factory::Register("CGNR", create_object);
}
#endif

//- parameter entries
namespace {
  void append_entry(Parameters& param)
  {
    param.Register_int("maximum_number_of_iteration", 0);
    param.Register_double("convergence_criterion_squared", 0.0);

    param.Register_string("verbose_level", "NULL");
  }


#ifdef USE_PARAMETERS_FACTORY
  bool init_param = ParametersFactory::Register("Solver.CGNR", append_entry);
#endif
}
//- end

//- parameters class
Parameters_Solver_CGNR::Parameters_Solver_CGNR() { append_entry(*this); }
//- end

const std::string Solver_CGNR::class_name = "Solver_CGNR";

//====================================================================
void Solver_CGNR::set_parameters(const Parameters& params)
{
  return this->Solver_CG::set_parameters(params);
}


//====================================================================
void Solver_CGNR::solve(Field& xq, const Field& b, int& Nconv, double& diff)
{
  Fopr   *fopr     = this->Solver_CG::get_fopr();
  string mode_prev = fopr->get_mode();

#pragma omp master
  {
    vout.detailed(m_vl, "%s: solver starts\n", class_name.c_str());
  }
#pragma omp barrier

  if (mode_prev == "DdagD") {
    this->Solver_CG::solve(xq, b, Nconv, diff);  // fallback to CG solver
    return;
  }

  if (!((mode_prev == "D") || (mode_prev == "Ddag"))) {
    vout.crucial(m_vl, "%s: unsupported mode for fopr: %s.", class_name.c_str(), mode_prev.c_str());
    abort();
  }

  //  Field b2(b);
#pragma omp master
  {
    if ((m_b2.nin() != b.nin()) || (m_b2.nvol() != b.nvol()) ||
        (m_b2.nex() != b.nex())) {
      m_b2.reset(b.nin(), b.nvol(), b.nex());
    }
  }
#pragma omp barrier

  fopr->mult_dag(m_b2, b);  // b2 = fopr->mult_dag(b);

#pragma omp master
  {
    if (mode_prev == "D") {
      fopr->set_mode("DdagD");
    } else if (mode_prev == "Ddag") {
      fopr->set_mode("DDdag");
    }
  }
#pragma omp barrier

  this->Solver_CG::solve(xq, m_b2, Nconv, diff);

#pragma omp master
  {
    fopr->set_mode(mode_prev);
  }
#pragma omp barrier
}


//====================================================================
//============================================================END=====
