/*!
        @file    $Id: test_Gauge_Plaquette.cpp #$

        @brief

        @author  Hideo Matsufuru <hideo.matsufuru@kek.jp> (matsufuru)
                 $LastChangedBy: aoym $

        @date    $LastChangedDate: 2013-01-22 13:51:53 #$

        @version $LastChangedRevision: 1144 $
*/

#include "test.h"

#include "gaugeConfig.h"
#include "staples.h"

#include "randomNumbers_Mseries.h"

//====================================================================
//! Test of gauge quantities.

/*!
                          [28 Dec 2011 H.Matsufuru]
    (Coding history will be recovered from trac.)
    YAML is implemented.  [14 Nov 2012 Y.Namekawa]
 */

namespace Test_Gauge {
  const std::string test_name = "Gauge.Plaquette";

  //- test-private parameters
  namespace {
    const std::string filename_input  = "test_Gauge.yaml";
    const std::string filename_output = "stdout";

    class Parameters_Test_Gauge : public Parameters {
     public:
      Parameters_Test_Gauge()
      {
        Register_string("gauge_config_status", "NULL");
        Register_string("gauge_config_type_input", "NULL");
        Register_string("config_filename_input", "NULL");

        Register_string("verbose_level", "NULL");

        Register_double("expected_result", 0.0);
      }
    };
  }

  //- prototype declaration
  int plaquette(void);

#ifdef USE_TESTMANAGER_AUTOREGISTER
  namespace {
#if defined(USE_GROUP_SU2)
    // Nc=2 is not available.
#else
    static const bool is_registered = TestManager::RegisterTest(
      test_name,
      plaquette
      );
#endif
  }
#endif

  //====================================================================
  int plaquette(void)
  {
    // #####  parameter setup  #####
    int Nvol = CommonParameters::Nvol();
    int Ndim = CommonParameters::Ndim();

    Parameters *params_test = new Parameters_Test_Gauge;
    Parameters *params_all  = new Parameters;

    params_all->Register_Parameters("Test_Gauge", params_test);

    ParameterManager_YAML params_manager;
    params_manager.read_params(filename_input, params_all);

    const string str_gconf_status = params_test->get_string("gauge_config_status");
    const string str_gconf_read   = params_test->get_string("gauge_config_type_input");
    const string readfile         = params_test->get_string("config_filename_input");
    const string str_vlevel       = params_test->get_string("verbose_level");

    const bool   do_check        = params_test->is_set("expected_result");
    const double expected_result = do_check ? params_test->get_double("expected_result") : 0.0;

    Bridge::VerboseLevel vl = vout.set_verbose_level(str_vlevel);

    //- print input parameters
    vout.general(vl, "  gconf_status = %s\n", str_gconf_status.c_str());
    vout.general(vl, "  gconf_read   = %s\n", str_gconf_read.c_str());
    vout.general(vl, "  readfile     = %s\n", readfile.c_str());
    vout.general(vl, "  vlevel       = %s\n", str_vlevel.c_str());
    vout.general(vl, "\n");

    //- input parameter check
    int err = 0;
    err += ParameterCheck::non_NULL(str_gconf_status);

    if (err) {
      vout.crucial(vl, "%s: Input parameters have not been set.\n", test_name.c_str());
      abort();
    }


    // ####  Set up a gauge configuration  ####
    Field_G     *U          = new Field_G(Nvol, Ndim);
    GaugeConfig *gconf_read = new GaugeConfig(str_gconf_read);

    if (str_gconf_status == "Continue") {
      gconf_read->read_file((Field *)U, readfile);
    } else if (str_gconf_status == "Cold_start") {
      U->set_unit();
    } else if (str_gconf_status == "Hot_start") {
      RandomNumbers_Mseries rand(1234567);
      U->set_random(&rand);
    } else {
      vout.crucial(vl, "%s: unsupported gconf status \"%s\".\n", test_name.c_str(), str_gconf_status.c_str());
      abort();
    }


    // #### object setup #####
    Staples *staple = new Staples;
    Timer   *timer  = new Timer(test_name);


    // ####  Execution main part  ####
    timer->start();

    double result = staple->plaquette(*U);
    vout.general(vl, "plaq = %.8f\n", result);

    timer->report();


    // ####  tydy up  ####
    delete params_test;
    delete params_all;

    delete timer;

    delete U;
    delete gconf_read;

    delete staple; // added by s.motoki[12/06/05].


    if (do_check) {
      return Test::verify(expected_result, result);
    } else {
      vout.detailed(vl, "check skipped: expected_result not set.\n\n");
      return EXIT_SKIP;
    }
  }
} // namespace Test_Gauge
