/*!
        @file    $Id: test_Rational_Approx.cpp #$

        @brief

        @author  Hideo Matsufuru <hideo.matsufuru@kek.jp> (matsufuru)
                 $LastChangedBy: aoym $

        @date    $LastChangedDate:: 2014-09-16 12:23:39 #$

        @version $LastChangedRevision: 1144 $
*/

#include "test.h"

#include "math_Rational.h"

//====================================================================
//! Test of rational approximation of fermion operators.

/*!
                                [28 Dec 2011 H.Matsufuru]
    (Coding history will be recovered from trac.)
    YAML is implemented.        [14 Nov 2012 Y.Namekawa]
 */

namespace Test_Rational {
  const std::string test_name = "Rational.Approx";

  //- test-private parameters
  namespace {
    const std::string filename_input  = "test_Rational_Approx.yaml";
    const std::string filename_output = "stdout";

    class Parameters_Test_Rational : public Parameters {
     public:
      Parameters_Test_Rational()
      {
        Register_string("verbose_level", "NULL");

        Register_double("expected_result", 0.0);
      }
    };
  }

  //- prototype declaration
  int approx(void);

#ifdef USE_TESTMANAGER_AUTOREGISTER
  namespace {
#if defined(USE_GROUP_SU2)
    // Nc=2 is not available.
#else
    static const bool is_registered = TestManager::RegisterTest(
      test_name,
      approx
      );
#endif
  }
#endif

  //====================================================================
  int approx(void)
  {
    // ####  parameter setup  ####
    Parameters *params_test     = new Parameters_Test_Rational;
    Parameters *params_rational = new Parameters_Math_Rational;
    Parameters *params_all      = new Parameters;

    params_all->Register_Parameters("Test_Rational", params_test);
    params_all->Register_Parameters("Math_Rational", params_rational);

    ParameterManager_YAML params_manager;
    params_manager.read_params(filename_input, params_all);

    const string str_vlevel = params_test->get_string("verbose_level");

    const bool   do_check        = params_test->is_set("expected_result");
    const double expected_result = do_check ? params_test->get_double("expected_result") : 0.0;

    const int    n_exp  = params_rational->get_int("exponent_numerator");
    const int    d_exp  = params_rational->get_int("exponent_denominator");
    const double x_min  = params_rational->get_double("lower_bound");
    const double x_max  = params_rational->get_double("upper_bound");
    const int    Npoint = params_rational->get_int("number_of_partitions");

    Bridge::VerboseLevel vl = vout.set_verbose_level(str_vlevel);

    //- print input parameters
    vout.general(vl, "  vlevel = %s\n", str_vlevel.c_str());
    vout.general(vl, "  n_exp  = %d\n", n_exp);
    vout.general(vl, "  d_exp  = %d\n", d_exp);
    vout.general(vl, "  x_min  = %12.8f\n", x_min);
    vout.general(vl, "  x_max  = %12.8f\n", x_max);
    vout.general(vl, "  Npoint = %d\n", Npoint);
    vout.general(vl, "\n");


    // ####  object setup  ####
    Math_Rational rational;
    rational.set_parameters(*params_rational);

    Timer *timer = new Timer(test_name);


    // ####  Execution main part  ####
    timer->start();

    double x     = x_min;
    double Dx    = (x_max - x_min) / Npoint;
    double p_exp = ((double)n_exp) / ((double)d_exp);

    for (int j = 0; j < Npoint + 1; ++j) {
      double y1 = rational.func(x);
      double y2 = pow(x, p_exp);

      vout.general(vl, "x,rational,x^(n/d) = %12.8f  %12.8f  %12.8f \n", x, y1, y2);

      x += Dx;
    }

    double result = x;

    timer->report();


    // ####  tidy up  ####
    delete params_test;
    delete params_rational;
    delete params_all;

    delete timer;


    if (do_check) {
      return Test::verify(expected_result, result);
    } else {
      vout.detailed(vl, "check skipped: expected_result not set.\n\n");
      return EXIT_SKIP;
    }
  }
} // namespace Test_Rational
